package ru.dbotthepony.kstarbound.defs.tile

import com.google.gson.GsonBuilder
import ru.dbotthepony.kstarbound.io.KConcreteTypeAdapter
import ru.dbotthepony.kvector.vector.Color

data class TileDefinition(
	val materialId: Int,
	val materialName: String,
	val particleColor: Color? = null,
	val itemDrop: String? = null,
	val description: String = "...",
	val shortdescription: String = "...",
	val footstepSound: String? = null,

	val blocksLiquidFlow: Boolean = true,
	val soil: Boolean = false,

	val health: Double = 0.0,
	val category: String,

	val renderTemplate: RenderTemplate,
	val renderParameters: RenderParameters,
) {
	companion object {
		val ADAPTER = KConcreteTypeAdapter.Builder(TileDefinition::class)
			.plain(
				TileDefinition::materialId,
				TileDefinition::materialName,
				TileDefinition::particleColor,
				TileDefinition::itemDrop,
				TileDefinition::description,
				TileDefinition::shortdescription,
				TileDefinition::footstepSound,
				TileDefinition::blocksLiquidFlow,
				TileDefinition::soil,
				TileDefinition::health,
				TileDefinition::category
			)
			.plain(TileDefinition::renderTemplate, RenderTemplate.CACHE)
			.plain(
				TileDefinition::renderParameters,
			)
			.build()

		fun registerGson(gsonBuilder: GsonBuilder) {
			ADAPTER.register(gsonBuilder)
		}
	}
}
